@@ -181,6 +181,8 @@ def mkDs(x):
             return mkTag(("CurrentDueFee",fns))
         case ("已付费用",*fns) | ("lastFeePaid",*fns):
             return mkTag(("LastFeePaid",fns))
+        case ("到期月份",bn) | ("monthsTillMaturity",bn):
+            return mkTag(("MonthsTillMaturity",bn))
         case ("Min", ds1, ds2):
             return mkTag(("Min",[mkDs(ds1),mkDs(ds2)]))
         case ("Max", ds1, ds2):
@@ -208,18 +210,42 @@ def isPre(x):
 
 
 def mkPre(p):
-    dealStatusMap = {"摊还":"Current"
-                     ,"加速清偿":"Accelerated"
-                     ,"循环":"Revolving"}
+    def isIntQuery(y):
+        match y:
+            case ("monthsTillMaturity",_):
+                return True 
+            case ("到期月份",_):
+                return True 
+            case _:
+                return False
+    dealStatusMap = {"摊还":"Current" ,"加速清偿":"Accelerated" ,"循环":"Revolving"}
+
     match p:
+        case [ds,"=",n] :
+            if isIntQuery(ds):
+                return mkTag(("IfEqInt",[mkDs(ds),n]))
+            else:
+                return mkTag(("IfEqBal",[mkDs(ds),n]))
         case [ds,">",amt]:
-            return mkTag(("IfGT",[mkDs(ds),amt]))
+            if isIntQuery(ds):
+                return mkTag(("IfGTInt",[mkDs(ds),amt]))
+            else:
+                return mkTag(("IfGT",[mkDs(ds),amt]))
         case [ds,"<",amt]:
-            return mkTag(("IfLT",[mkDs(ds),amt]))
+            if isIntQuery(ds):
+                return mkTag(("IfLTInt",[mkDs(ds),amt]))
+            else:
+                return mkTag(("IfLT",[mkDs(ds),amt]))
         case [ds,">=",amt]:
-            return mkTag(("IfGET",[mkDs(ds),amt]))
+            if isIntQuery(ds):
+                return mkTag(("IfGETInt",[mkDs(ds),amt]))
+            else:
+                return mkTag(("IfGET",[mkDs(ds),amt]))
         case [ds,"<=",amt]:
-            return mkTag(("IfLET",[mkDs(ds),amt]))
+            if isIntQuery(ds):
+                return mkTag(("IfLETInt",[mkDs(ds),amt]))
+            else:
+                return mkTag(("IfLET",[mkDs(ds),amt]))
         case [ds,"=",0]:
             return mkTag(("IfZero",mkDs(ds)))
         case [">",_d]:
@@ -369,13 +395,14 @@ def mkBnd(bn,x):
               ,"startDate": originDate
               ,"rateType": bndInterestInfo
               ,"bondType": bndType}:
+            md = x.get("到期日",None) or x.get("maturityDate",None)
             return {bn: {"bndName": bn
                         ,"bndBalance": bndBalance
                           , "bndRate": bndRate
                           , "bndOriginInfo":
                            {"originBalance": originBalance
-                               , "originDate": originDate
-                               , "originRate": originRate}
+                            , "originDate": originDate
+                            , "originRate": originRate} | {"maturityDate": md}
                           , "bndInterestInfo": mkBondRate(bndInterestInfo)
                           , "bndType": mkBondType(bndType)
                           , "bndDuePrin": 0
@@ -520,6 +547,32 @@ def mkStatus(x):
         case _:
             raise RuntimeError(f"Failed to match :{x}:mkStatus")
 
+def readStatus(x, locale):
+    m = {"en":{'amort':"Amortizing"
+               ,'def':"Defaulted"
+               ,'acc':"Accelerated"
+               ,'end':"Ended"
+               ,'pre':"PreClosing" }
+        ,"cn":{'amort':"摊销"
+               ,'def':"违约"
+               ,'acc':"加速清偿"
+               ,'end':"结束"
+               ,'pre':"设计" }}
+    match x:
+        case {"tag":"Amortizing"}:
+            return m[locale]['amort']
+        case {"tag":"DealAccelerated"}:
+            return m[locale]['acc']
+        case {"tag":"DealDefaulted"}:
+            return m[locale]['def']
+        case {"tag":"Ended"}:
+            return m[locale]['end']
+        case {"tag":"PreClosing"}:
+            return m[locale]['pre']
+        case _:
+            raise RuntimeError(f"Failed to read deal status:{x} with locale: {locale}")
+
+
 def mkWhenTrigger(x):
     match x:
         case "回收后"|"BeforeCollect":
@@ -549,14 +602,26 @@ def mkThreshold(x):
 
 def _rateTypeDs(x):
     h = x[0]
-    if h in set(["资产池累积违约率","cumPoolDefaultedRate",
-    "债券系数","bondFactor",
-    "资产池系数","poolFactor"]):
+    if h in set(["资产池累积违约率","cumPoolDefaultedRate"
+                 ,"债券系数","bondFactor"
+                 ,"资产池系数","poolFactor"]):
         return True
     return False
     
 def mkTrigger(x):
     match x : 
+        case [">",_d]:
+            return mkTag(("AfterDate",_d))
+        case [">=",_d]:
+            return mkTag(("AfterOnDate",_d))
+        case ["到期日未兑付",_bn] | ["passMaturity",_bn]:
+            return mkTag(("PassMaturityDate",_bn))
+        case ["所有满足",*trgs] | ["all",*trgs]:
+            return mkTag(("AllTrigger",[ mkTrigger(t) for t in trgs ]))
+        case ["任一满足",*trgs] | ["any",*trgs]:
+            return mkTag(("AnyTrigger",[ mkTrigger(t) for t in trgs ]))
+        case ["一直",b] | ["always",b]:
+            return mkTag(("Always",b))
         case [ds,cmp,v] if (isinstance(v,float) and _rateTypeDs(ds)):
             return mkTag(("ThresholdRate",[mkThreshold(cmp),mkDs(ds),v]))
         case [ds,cmp,ts] if _rateTypeDs(ds):
@@ -565,22 +630,6 @@ def mkTrigger(x):
             return mkTag(("ThresholdBal",[mkThreshold(cmp),mkDs(ds),v]))
         case [ds,cmp,ts]:
             return mkTag(("ThresholdBalCurve",[mkThreshold(cmp),mkDs(ds),mkTs("ThresholdCurve",ts)]))
-        case [">",_d]:
-            return mkTag(("AfterDate",_d))
-        case [">=",_d]:
-            return mkTag(("AfterOnDate",_d))
-        case ["到期日未兑付",_bn] | ["passMaturity",_bn]:
-            return mkTag(("PassMaturityDate",_bn))
-        #case ("目标摊还不足",bn):
-        #   return mkTag(("PrinShortfall",bn))
-        #case ["到期日未兑付",bn]:
-        #    return mkTag(("MissMatureDate",bn))
-        case ["所有满足",*trgs] | ["all",*trgs]:
-            return mkTag(("AllTrigger",[ mkTrigger(t) for t in trgs ]))
-        case ["任一满足",*trgs] | ["any",*trgs]:
-            return mkTag(("AnyTrigger",[ mkTrigger(t) for t in trgs ]))
-        case ["一直",b] | ["always",b]:
-            return mkTag(("Always",b))
         case _:
             raise RuntimeError(f"Failed to match :{x}:mkTrigger")
 
@@ -989,4 +1038,35 @@ def readPricingResult(x, locale)->dict:
     else:
         raise RuntimeError(f"Failed to read princing result: {x} with tag={tag}")
 
-    return pd.DataFrame.from_dict({k:v['contents'] for k,v in x.items()} , orient='index', columns=h[locale]).sort_index()
\ No newline at end of file
+    return pd.DataFrame.from_dict({k:v['contents'] for k,v in x.items()} , orient='index', columns=h[locale]).sort_index()
+
+def readRunSummary(x, locale)->dict:
+
+    r = {}
+    if x is None:
+        return None
+    
+    bndStatus ={'cn':["本金违约","利息违约","起算余额"],'en':["Balance Defaults","Interest Defaults","Original Balance"]}
+    bond_defaults = [ (_['contents'][0],_['tag'], _['contents'][1], _['contents'][2]) 
+                        for _ in x if _['tag'] in set(['BondOutstanding','BondOutstandingInt']) ]
+    _fmap = {"cn":{'BondOutstanding':"本金违约","BondOutstandingInt":"利息违约"}
+            ,"en":{'BondOutstanding':"Balance Defaults","BondOutstandingInt":"Interest Defaults"}}
+    bndNames = set([ y[0] for y in bond_defaults])
+    bndSummary = pd.DataFrame(columns=bndStatus[locale],index=list(bndNames))
+    for bn,amt_type,amt,begBal in bond_defaults:
+       bndSummary.loc[bn][_fmap[locale][amt_type]] = amt
+       bndSummary.loc[bn][bndStatus[locale][2]] = begBal
+    bndSummary.fillna(0,inplace=True)
+    bndSummary["Total"] = bndSummary[bndStatus[locale][0]] + bndSummary[bndStatus[locale][1]]
+
+    r['bonds'] = bndSummary
+
+    dealStatusLog = {'cn':["日期","旧状态","新状态"],'en':["Date","From","To"]}
+    status_change_logs = [ (_['contents'][0],readStatus(_['contents'][1], locale),readStatus(_['contents'][2], locale)) 
+                            for _ in x if _['tag'] in set(['DealStatusChangeTo'])]
+    r['status'] = pd.DataFrame(data=status_change_logs,columns=dealStatusLog[locale])
+
+    return r
+
+
+
