@@ -4,15 +4,9 @@ import itertools,re
 from enum import Enum
 import numpy as np
 from functools import reduce
+from pyxirr import xirr,xnpv
 
-def query(d,p):
-    if len(p)==1:
-        return d[p[0]]
-    else:
-        if p[0] in d:
-            return query(d[p[0]],p[1:])
-        else:
-            return None
+from absbox.local.base import *
 
 def flat(xss) -> list:
     return reduce(lambda xs, ys: xs + ys, xss)
@@ -34,17 +28,6 @@ def mkTs(n, vs):
     return mkTag((n, vs))
 
 
-def backFillBal(x,ds):
-    b = pd.DataFrame({"日期":ds})
-    b.set_index("日期",inplace=True)
-    base = pd.concat([b,x],axis=1).sort_index()
-    paidOffDate = base[base['余额']==0].index[0]
-    base['flag'] = (base.index >= paidOffDate)
-    base.loc[base['flag']==True,"余额"]=0
-    base.loc[base['flag']==False,"余额"]= (base["余额"] + base["本金"]).shift(-1).fillna(method='bfill')
-    return base
-
-
 def unify(xs, ns):
     index_name = xs[0].index.name
     dfs = []
@@ -60,11 +43,21 @@ def backFillBal(x,ds):
     b = pd.DataFrame({"日期": ds})
     b.set_index("日期", inplace=True)
     base = pd.concat([b, x], axis=1).sort_index()
-    paidOffDate = base[base['余额']==0].index[0]
-    base['flag'] = (base.index >= paidOffDate)
-    base.loc[base['flag']==True, "余额"] = 0
-    base.loc[base['flag']==False, "余额"] = (base["余额"] + base["本金"]).shift(-1).fillna(method='bfill')
-    return base.drop(["flag"], axis=1)
+    paidOffDate = None
+    r = None
+    if any(base['余额']==0):
+        paidOffDate = base[base['余额']==0].index[0]
+        base['flag'] = (base.index >= paidOffDate)
+        base.loc[base['flag']==True, "余额"] = 0
+        base.loc[base['flag']==False, "余额"] = (base["余额"] + base["本金"]).shift(-1).fillna(method='bfill')
+        r = base.drop(["flag"], axis=1)
+    else:
+        last_index = base.index.to_list()[-1]
+        last_keep_balance = base.at[last_index, "余额"]
+        base["余额"] = (base["余额"] + base["本金"]).shift(-1).fillna(method='bfill')
+        base.at[last_index, "余额"] = last_keep_balance
+        r = base
+    return r 
 
 
 def bondView(r,flow=None, flowName=True,flowDates=None,rnd=2):
@@ -86,6 +79,7 @@ def bondView(r,flow=None, flowName=True,flowDates=None,rnd=2):
             result.append(backFillBal(bnd,flowDates)[flow])
         else:
             result.append(backFillBal(bnd,flowDates))
+    
     x = pd.concat(result,axis=1)
     bnd_cols_count = len(flow) if flow else default_bnd_col_size
     headers = [ bnd_cols_count*[bn] for bn in bnd_names]
@@ -145,8 +139,10 @@ def balanceSheetView(r, ds=None, equity=None, rnd=2):
     bv = bondView(r, flow=["余额"],flowDates=ds,flowName=False)
     av = accView(r, flow=["余额"],flowName=False)
 
-    r['pool']["flow"]["不良"] = r['pool']['flow']["违约金额"].cumsum() - r['pool']['flow']["回收金额"].cumsum()
-    pv = r['pool']['flow'][["未偿余额","不良"]]
+    pv = r['pool']['flow'][["未偿余额"]]
+    if "违约金额" in r['pool']['flow'] and "回收金额" in r['pool']['flow']:
+        r['pool']["flow"]["不良"] = r['pool']['flow']["违约金额"].cumsum() - r['pool']['flow']["回收金额"].cumsum()
+        pv = r['pool']['flow'][["未偿余额","不良"]]
     if equity:
         equityFlow = bondView(r, flow=["本息合计"],flowDates=ds,flowName=False)[equity]
         equityFlow.columns = pd.MultiIndex.from_arrays([["权益"]*len(equity), list(equityFlow.columns)])
@@ -178,47 +174,6 @@ def balanceSheetView(r, ds=None, equity=None, rnd=2):
         else:
             bs["权益", "合计"] = bs["资产", "合计"] - bs["负债", "合计"] 
 
-    # build PnL
-     #   pool_index = r['pool']['flow'].index
-     #   agg_flag = bs.index.get_indexer(list(pool_index),method='ffill')
-     #   pool_cpy = r['pool']['flow'].copy(deep=True)
-     #   pool_cpy['flag'] = agg_flag
-     #   pool_cpy = pool_cpy.groupby("flag", sort=False).aggregate(np.sum)
-     #   new_index_length = min(len(pool_cpy.index), len(bs.index))
-     #   #pool_cpy.index = bs.index[:len(pool_cpy.index)]
-     #   print(pool_cpy)
-     #   print(bs)
-     #   pool_cpy.index = bs.index[:new_index_length]
-
-     #   poolIntflow = pool_cpy['利息']
-     #   poolPrinflow = pool_cpy['本金']
-     #   poolPpyflow = pool_cpy['早偿金额']
-     #   poolRecflow = pool_cpy['回收金额']
-
-     #   feeFlow = feeView(r,flow="支付")
-     #   feeFlow["合计"] = feeFlow.sum(axis=1)
-     #   agg_flag = bs.index.get_indexer(list(feeFlow.index),method='bfill')
-     #   feeFlow['flag'] = agg_flag
-     #   feeFlow.groupby("flag").aggregate(np.sum)
-     #   feeFlow.index = bs.index
-     #   
-     #   liability_p_v = bondView(r,flow="本金")
-     #   liability_p_v['合计'] = liability_p_v.sum(axis=1)
-     #   liability_i_v = bondView(r,flow="利息")
-     #   liability_i_v['合计'] = liability_i_v.sum(axis=1)
-
-     #   bs["收入","本金"] = poolPrinflow
-     #   bs["收入","利息"] = poolIntflow
-     #   bs["收入","早偿"] = poolPpyflow
-     #   bs["收入","回收"] = poolRecflow
-
-     #   bs["支出","负债-本金"] = liability_p_v['合计']
-     #   bs["支出","负债-利息"] = liability_i_v['合计']
-     #   bs["支出","负债-费用"] = feeFlow['合计']
-     #   
-     #   bs["利润","合计"] = bs["收入","本金"] + bs["收入","利息"] + bs["收入","早偿"] + bs["收入","回收"]
-     #   bs["利润","合计"] = bs["利润","合计"] - bs["支出","负债-本金"] - bs["支出","负债-利息"] - bs["支出","负债-费用"]
-
     except RuntimeError as e:              
         print(f"Error: 其他错误=>{e}")      
     
@@ -239,6 +194,83 @@ def aggStmtByDate(s):
     return s.groupby("日期").sum()
 
 
+def aggCFby(_df, interval, cols):
+    df = _df.copy()
+    idx = None
+    dummy_col = '_index'
+    df[dummy_col] = df.index
+    _mapping = {"月份":"M"
+               ,"Month":"M"
+               ,"M":"M"
+               ,"month":"M"}
+    if df.index.name == "日期":
+        idx = "日期"
+    else:
+        idx = "date"
+    df[dummy_col]=pd.to_datetime(df[dummy_col]).dt.to_period(_mapping[interval])
+    return df.groupby([dummy_col])[cols].sum().rename_axis(idx)#.drop(columns=[dummy_col])
+
+
+def irr(flow,init=None):
+    def extract_cash_col(_cols):
+        if _cols == china_bondflow_fields_s:
+            return flow['本息合计']
+        elif _cols == english_bondflow_fields_s: 
+            return flow['cash']
+        else:
+            raise RuntimeError("Failed to match",_cols)
+
+    cols = flow.columns.to_list()
+    dates = flow.index.to_list()
+    amounts = extract_cash_col(cols).to_list()
+        
+    if init is not None:
+        invest_date,invest_amount = init
+        dates = [invest_date]+dates
+        amounts = [invest_amount]+amounts
+    
+    return xirr(np.array(dates), np.array(amounts))
+
+
+def sum_fields_to_field(_df,cols,col):
+    df = _df.copy()
+    df[col] = df[cols].sum(axis=1)
+    return df
+
+
+def npv(_flow,**p):
+    flow = _flow.copy()
+    cols = flow.columns.to_list()
+    idx_name = flow.index.name
+    init_date,_init_amt = p['init']
+    init_amt = _init_amt if _init_amt!=0.00 else 0.0001
+    def _pv(_af):
+        af = flow[_af]
+        return xnpv(p['rate'],[init_date]+flow.index.to_list(),[-1*init_amt]+af.to_list())
+    match (cols,idx_name):
+        case (china_rental_flow,"日期"):
+            return _pv("租金")
+        case (english_rental_flow,"Date"):
+            return _pv("Rental")
+        case (english_mortgage_flow_fields,"Date"):
+            sum_fields_to_field(flow,["Principal","Interest","Prepayment","Recovery"], "Cash")
+            return _pv("Cash")
+        case (china_bondflow_fields,"日期"):
+            return _pv("本息合计")
+        case (english_bondflow_fields,"Date"):
+            return _pv("cash")
+        case _:
+            raise RuntimeError("Failed to match",cols,idx_name)
+
+
+def update_deal(d,i,c):
+    "A patch function to update a deal data list in immuntable way"
+    _d = d.copy()
+    _d.pop(i)
+    _d.insert(i,c)
+    return _d
+
+
 class DC(Enum):  # TODO need to check with HS code
     DC_30E_360 = "DC_30E_360"
     DC_30Ep_360 ="DC_30Ep_360"
@@ -251,3 +283,9 @@ class DC(Enum):  # TODO need to check with HS code
     DC_30_360_ISDA = "DC_30_360_ISDA"
     DC_30_360_German = "DC_30_360_German"
     DC_30_360_US  = "DC_30_360_US"
+
+def str2date(x:str):
+    return datetime.strptime(x, '%Y-%m-%d').date()
+
+def daysBetween(sd,ed):
+    return (ed - sd).days
\ No newline at end of file
